#!/usr/bin/env ruby
# $Id: tc_ext_member.rb,v 1.7 2004/04/21 16:23:03 eto Exp $
# Copyright (C) 2003-2004 Kouichirou Eto, All rights reserved.
# This is free software with ABSOLUTELY NO WARRANTY.
# You can redistribute it and/or modify it under the terms of the GNU GPL2.

$LOAD_PATH.unshift('../../lib')
$LOAD_PATH.unshift('../../compat')
require 'test/unit'
require 'qwik/server'

class TC_PasswordGenerator < Test::Unit::TestCase
  def setup()
    @config = Qwik::Config.new(true)
    @config.pass_file = './password.txt'
  end

  def test_password
    pw = Qwik::PasswordGenerator.new(@config)
    assert_equal("E4268F8D", pw.generate("test@example.com"))
    assert_equal("37CF90D4", pw.generate("test@example.com", 1))
  end
end

class TC_PasswordMailGenerator < Test::Unit::TestCase
  def setup()
    @config = Qwik::Config.new(true)
    @config.pass_file = './password.txt'
    @config.data_dir = './'
  end

  def test_getpass_mail
    with_dir('test'){
      site = Qwik::Site.new(@config, 'test')
      pmgen = Qwik::PasswordMailGenerator.new(@config, site)
      mail = pmgen.generate('test@example.com')
      assert_equal("test@qwik.jp", mail.from)
      assert_equal('test@example.com', mail.to)
      assert_equal('http://qwik.jp/ におけるパスワード', mail.subject)
      assert_equal(<<"EOT", mail.content)
http://qwik.jp/ におけるユーザー名とパスワードは、下記の通りです。

ユーザー名:	test@example.com
パスワード:	E4268F8D

http://qwik.jp/test/ に再度アクセスしてみてください。
EOT
    }
    with_dir('example.com'){
      site = Qwik::Site.new(@config, 'example.com')
      pmgen = Qwik::PasswordMailGenerator.new(@config, site)
      mail = pmgen.generate('test@example.com')
      assert_equal("info@example.com", mail.from)
      assert_equal('test@example.com', mail.to)
      assert_equal('http://example.com/ におけるパスワード', mail.subject)
      assert_equal(<<"EOT", mail.content)
http://example.com/ におけるユーザー名とパスワードは、下記の通りです。

ユーザー名:	test@example.com
パスワード:	E4268F8D

http://example.com/ に再度アクセスしてみてください。
EOT
    }
  end

  def with_dir(sitename)
    dir = ('./'+sitename+'/').path
    dir.rmtree if dir.directory?
    dir.rmdir  if dir.directory?
    dir.mkdir if !dir.directory?
    yield
    dir.rmtree if dir.directory?
    dir.rmdir  if dir.directory?
  end
end
